# Copyright 2020, Microsoft Corporation
# Copyright 2020, Chris PeBenito <pebenito@ieee.org>
#
# This file is part of SETools.
#
# SETools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1 of
# the License, or (at your option) any later version.
#
# SETools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with SETools.  If not, see
# <http://www.gnu.org/licenses/>.
#

import os
import logging
import unittest

from .. import mixins
from ..policyrep.util import compile_policy

from setools import TERuletype
from setools.checker.assertte import AssertTE
from setools.exception import InvalidCheckValue, InvalidCheckOption


class AssertTETest(mixins.ValidateRule, unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.p = compile_policy("tests/checker/assertte.conf")

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls.p.path)

    def test_unconfigured(self):
        """Test unconfigured."""
        with self.assertRaises(InvalidCheckValue):
            config = {}
            check = AssertTE(self.p, "test_unconfigured", config)

    def test_invalid_option(self):
        """Test invalid option"""
        with self.assertRaises(InvalidCheckOption):
            config = {"INVALID": "option"}
            check = AssertTE(self.p, "test_invalid_option", config)

    def test_source(self):
        """Test source setting."""
        config = {"source": "src"}
        check = AssertTE(self.p, "test_source", config)

        expected = self.p.lookup_type("src")
        self.assertEqual(expected, check.source)

    def test_source_fail(self):
        """Test source setting failure."""
        with self.assertRaises(InvalidCheckValue):
            config = {"source": "FAIL"}
            check = AssertTE(self.p, "test_source_fail", config)

    def test_target(self):
        """Test target setting."""
        config = {"target": "tgt"}
        check = AssertTE(self.p, "test_target", config)

        expected = self.p.lookup_type("tgt")
        self.assertEqual(expected, check.target)

    def test_target_fail(self):
        """Test target setting failure."""
        with self.assertRaises(InvalidCheckValue):
            config = {"target": "FAIL2"}
            check = AssertTE(self.p, "test_target_fail", config)

    def test_exempt_source(self):
        """Test exempt_source setting."""
        config = {"source": "system",
                  "exempt_source": " exempt_src1   exempt_src2 "}
        check = AssertTE(self.p, "test_exempt_source", config)

        # exempt_src2 is an attr
        expected = set((self.p.lookup_type("exempt_src1"),
                        self.p.lookup_type("exempt_source_type")))
        self.assertIsInstance(check.exempt_source, frozenset)
        self.assertSetEqual(expected, check.exempt_source)

    def test_source_missing_ignored(self):
        """Test exempt_source missing type is ignroed."""
        config = {"source": "system",
                  "exempt_source": "FAIL  exempt_src2"}
        check = AssertTE(self.p, "test_source_missing_ignored", config)

        # exempt_src2 is an attr
        expected = set((self.p.lookup_type("exempt_source_type"),))
        self.assertIsInstance(check.exempt_source, frozenset)
        self.assertSetEqual(expected, check.exempt_source)

    def test_exempt_target(self):
        """Test exempt_target setting."""
        config = {"target": "system",
                  "exempt_target": " exempt_tgt1   exempt_tgt2 "}
        check = AssertTE(self.p, "test_exempt_target", config)

        # exempt_tgt2 is an attr
        expected = set((self.p.lookup_type("exempt_tgt1"),
                        self.p.lookup_type("exempt_target_type")))
        self.assertIsInstance(check.exempt_target, frozenset)
        self.assertSetEqual(expected, check.exempt_target)

    def test_target_missing_ignored(self):
        """Test exempt_target missing type is ignroed."""
        config = {"target": "system",
                  "exempt_target": "FAIL  exempt_tgt2"}
        check = AssertTE(self.p, "test_target_missing_ignored", config)

        # exempt_tgt2 is an attr
        expected = set((self.p.lookup_type("exempt_target_type"),))
        self.assertIsInstance(check.exempt_target, frozenset)
        self.assertSetEqual(expected, check.exempt_target)

    def test_expect_source(self):
        """Test expect_source setting."""
        with self.subTest("Success"):
            config = {"tclass": "infoflow3",
                      "expect_source": " exempt_src1   exempt_src2 "}
            check = AssertTE(self.p, "test_expect_source", config)

            # exempt_src2 is an attr
            expected = set((self.p.lookup_type("exempt_src1"),
                            self.p.lookup_type("exempt_source_type")))
            self.assertIsInstance(check.expect_source, frozenset)
            self.assertSetEqual(expected, check.expect_source)

        with self.subTest("Failure"):
            with self.assertRaises(InvalidCheckValue):
                config = {"tclass": "infoflow3",
                          "expect_source": " source1   INVALID "}
                check = AssertTE(self.p, "test_expect_source_fail", config)

    def test_expect_target(self):
        """Test expect_target setting."""
        with self.subTest("Success"):
            config = {"tclass": "infoflow3",
                      "expect_target": " exempt_tgt1   exempt_tgt2 "}
            check = AssertTE(self.p, "test_expect_target", config)

            # exempt_tgt2 is an attr
            expected = set((self.p.lookup_type("exempt_tgt1"),
                            self.p.lookup_type("exempt_target_type")))
            self.assertIsInstance(check.expect_target, frozenset)
            self.assertSetEqual(expected, check.expect_target)

        with self.subTest("Failure"):
            with self.assertRaises(InvalidCheckValue):
                config = {"tclass": "infoflow3",
                          "expect_target": " target1   INVALID "}
                check = AssertTE(self.p, "test_expect_target_fail", config)

    def test_tclass(self):
        """Test tclass setting."""
        config = {"tclass": "infoflow3  infoflow2"}
        check = AssertTE(self.p, "test_tclass", config)

        expected = set((self.p.lookup_class("infoflow3"),
                        self.p.lookup_class("infoflow2")))
        self.assertEqual(expected, check.tclass)

    def test_tclass_fail(self):
        """Test tclass setting failure."""
        with self.assertRaises(InvalidCheckValue):
            config = {"tclass": "FAIL_class"}
            check = AssertTE(self.p, "test_tclass_fail", config)

    def test_perms(self):
        """Test perms setting."""
        config = {"perms": " hi_w  super_r "}
        check = AssertTE(self.p, "test_perms", config)

        expected = set(("hi_w", "super_r"))
        self.assertEqual(expected, check.perms)

    def test_perms_fail(self):
        """Test perms setting failure."""
        with self.assertRaises(InvalidCheckValue):
            config = {"perms": "FAIL_perms"}
            check = AssertTE(self.p, "test_perms_fail", config)

    def test_check_passes(self):
        """Test the check passes, no matches"""
        config = {"perms": "null"}
        check = AssertTE(self.p, "test_check_passes", config)
        self.assertFalse(check.run())

    def test_check_passes_empty_source(self):
        """Test the check passes, empty source attribute"""
        config = {"tclass": "infoflow7",
                  "perms": "super_w"}
        check = AssertTE(self.p, "test_check_passes_empty_source", config)
        self.assertFalse(check.run())

    def test_check_passes_empty_target(self):
        """Test the check passes, empty target attribute"""
        config = {"tclass": "infoflow7",
                  "perms": "super_r"}
        check = AssertTE(self.p, "test_check_passes_empty_target", config)
        self.assertFalse(check.run())

    def test_check_passes_exempt_source_type(self):
        """Test the check passes, exempt_source_type"""
        config = {"tclass": "infoflow6",
                  "perms": "hi_w",
                  "exempt_source": "source1"}
        check = AssertTE(self.p, "test_check_passes_exempt_source_type", config)
        self.assertFalse(check.run())

    def test_check_passes_exempt_source_attr(self):
        """Test the check passes, exempt_source_attr"""
        config = {"tclass": "infoflow6",
                  "perms": "hi_r",
                  "exempt_source": "all_sources"}
        check = AssertTE(self.p, "test_check_passes_exempt_source_attr", config)
        self.assertFalse(check.run())

    def test_check_passes_exempt_target_type(self):
        """Test the check passes, exempt_target_type"""
        config = {"tclass": "infoflow5",
                  "perms": "low_w",
                  "exempt_source": "source1"}
        check = AssertTE(self.p, "test_check_passes_exempt_target_type", config)
        self.assertFalse(check.run())

    def test_check_passes_exempt_target_attr(self):
        """Test the check passes, exempt_target_attr"""
        config = {"tclass": "infoflow5",
                  "perms": "low_r",
                  "exempt_target": "all_targets"}
        check = AssertTE(self.p, "test_check_passes_exempt_target_attr", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_source(self):
        """Test the check passes, expect_source"""
        config = {"tclass": "infoflow6",
                  "perms": "hi_r",
                  "expect_source": "source1 source2"}
        check = AssertTE(self.p, "test_check_passes_expect_source", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_source_attr(self):
        """Test the check passes, expect_source with attribute"""
        config = {"tclass": "infoflow4",
                  "perms": "med_w",
                  "expect_source": "all_sources"}
        check = AssertTE(self.p, "test_check_passes_expect_source_attr", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_target(self):
        """Test the check passes, expect_target"""
        config = {"tclass": "infoflow6",
                  "perms": "hi_r",
                  "expect_target": "target1 target2"}
        check = AssertTE(self.p, "test_check_passes_expect_target", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_target_attr(self):
        """Test the check passes, expect_target with attribute"""
        config = {"tclass": "infoflow4",
                  "perms": "med_w",
                  "expect_target": "all_targets"}
        check = AssertTE(self.p, "test_check_passes_expect_target_attr", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_exempt_source(self):
        """"Test the check passes with both expected and exempted sources."""
        config = {"tclass": "infoflow5",
                  "perms": "low_r",
                  "expect_source": "source1",
                  "exempt_source": "source2"}
        check = AssertTE(self.p, "test_check_passes_expect_exempt_source", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_exempt_target(self):
        """"Test the check passes with both expected and exempted targets."""
        config = {"tclass": "infoflow5",
                  "perms": "low_r",
                  "expect_source": "source1",
                  "exempt_source": "source2"}
        check = AssertTE(self.p, "test_check_passes_expect_exempt_target", config)
        self.assertFalse(check.run())

    def test_check_fails(self):
        """Test the check fails"""
        with open("/dev/null", "w") as fd:
            config = {"tclass": "infoflow4",
                      "perms": "med_w",
                      "exempt_source": "source1",
                      "exempt_target": "target2"}
            check = AssertTE(self.p, "test_check_passes_exempt_target_attr", config)
            check.output = fd
            result = check.run()
            self.assertEqual(1, len(result), msg=result)
            self.validate_rule(result[0], TERuletype.allow, "source3", "target3", "infoflow4",
                               set(["med_w"]))

    def test_check_fails_expect_source(self):
        """Test the check fails, expect_source"""
        config = {"tclass": "infoflow7",
                  "perms": "super_w",
                  "expect_source": "source1"}
        check = AssertTE(self.p, "test_check_fails_expect_source", config)
        result = check.run()
        self.assertEqual(1, len(result), msg=result)
        self.assertIn("source1", result[0])

    def test_check_fails_expect_target(self):
        """Test the check fails, expect_target"""
        config = {"tclass": "infoflow7",
                  "perms": "super_r",
                  "expect_target": "target2"}
        check = AssertTE(self.p, "test_check_fails_expect_target", config)
        result = check.run()
        self.assertEqual(1, len(result), msg=result)
        self.assertIn("target2", result[0])



# Copyright 2020, Microsoft Corporation
#
# This file is part of SETools.
#
# SETools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1 of
# the License, or (at your option) any later version.
#
# SETools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with SETools.  If not, see
# <http://www.gnu.org/licenses/>.
#

import os
import unittest

from ..policyrep.util import compile_policy

from setools.checker.emptyattr import EmptyTypeAttr
from setools.exception import InvalidCheckOption, InvalidCheckValue


class EmptyTypeAttrTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.p = compile_policy("tests/checker/emptyattr.conf")

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls.p.path)

    def test_invalid_option(self):
        """Test invalid option"""
        with self.assertRaises(InvalidCheckOption):
            config = {"INVALID": "option"}
            check = EmptyTypeAttr(self.p, "test_invalid_option", config)

    def test_attr_setting(self):
        """EmptyTypeAttr test attr setting."""
        config = {"attr": "test1"}
        check = EmptyTypeAttr(self.p, "test_attr_setting", config)

        expected = self.p.lookup_typeattr("test1")
        self.assertEqual(expected, check.attr)

    def test_attr_setting_fail(self):
        """EmptyTypeAttr test attr setting with invalid attr."""
        with self.assertRaises(InvalidCheckValue):
            config = {"attr": "FAILATTR"}
            check = EmptyTypeAttr(self.p, "test_attr_setting_fail", config)

    def test_attr_setting_missing(self):
        """EmptyTypeAttr test attr setting missing."""
        with self.assertRaises(InvalidCheckValue):
            config = {}
            check = EmptyTypeAttr(self.p, "test_attr_setting_missing", config)

    def test_missingok_setting(self):
        """EmptyTypeAttr test missing_ok setting."""
        config = {"attr": "test1",
                  "missing_ok": "true"}
        check = EmptyTypeAttr(self.p, "test_missingok_setting", config)
        self.assertTrue(check.missing_ok)

        config = {"attr": "test1",
                  "missing_ok": " YeS "}
        check = EmptyTypeAttr(self.p, "test_missingok_setting", config)
        self.assertTrue(check.missing_ok)

        config = {"attr": "test1",
                  "missing_ok": " 1 "}
        check = EmptyTypeAttr(self.p, "test_missingok_setting", config)
        self.assertTrue(check.missing_ok)

        config = {"attr": "test1",
                  "missing_ok": " No "}
        check = EmptyTypeAttr(self.p, "test_missingok_setting", config)
        self.assertFalse(check.missing_ok)

    def test_pass(self):
        """EmptyTypeAttr test pass."""
        with open("/dev/null", "w") as fd:
            config = {"attr": "test1"}
            check = EmptyTypeAttr(self.p, "test_pass", config)
            check.output = fd
            result = check.run()
            self.assertEqual(0, len(result))

    def test_pass_missingok(self):
        """EmptyTypeAttr test pass by missing."""
        with open("/dev/null", "w") as fd:
            config = {"attr": "test2",
                      "missing_ok": "true"}
            check = EmptyTypeAttr(self.p, "test_pass_missingok", config)
            check.output = fd
            result = check.run()
            self.assertEqual(0, len(result))

    def test_fail(self):
        """EmptyTypeAttr test fail."""
        with open("/dev/null", "w") as fd:
            # also verify missing_ok doesn't induce a pass
            # when the attr exists
            config = {"attr": "test3",
                      "missing_ok": "true"}
            check = EmptyTypeAttr(self.p, "test_fail", config)
            check.output = fd
            result = check.run()
            expected = [self.p.lookup_type("test3_hit1"),
                        self.p.lookup_type("test3_hit2")]
            self.assertListEqual(expected, result)



# Copyright 2020, Microsoft Corporation
#
# This file is part of SETools.
#
# SETools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1 of
# the License, or (at your option) any later version.
#
# SETools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with SETools.  If not, see
# <http://www.gnu.org/licenses/>.
#

import os
import logging
import unittest


from setools.checker import util


class CheckerUtilTest(unittest.TestCase):

    def test_config_bool_value(self):
        """Test config_bool_value"""
        self.assertTrue(util.config_bool_value(" TrUe "))
        self.assertTrue((util.config_bool_value(" 1 ")))
        self.assertTrue((util.config_bool_value(" YeS ")))
        self.assertFalse((util.config_bool_value(" FalsE ")))
        self.assertFalse((util.config_bool_value(" 0 ")))
        self.assertFalse((util.config_bool_value(" No ")))

        self.assertTrue(util.config_bool_value(True))
        self.assertFalse((util.config_bool_value(None)))
        self.assertFalse((util.config_bool_value(False)))



# Copyright 2020, Microsoft Corporation
#
# This file is part of SETools.
#
# SETools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1 of
# the License, or (at your option) any later version.
#
# SETools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with SETools.  If not, see
# <http://www.gnu.org/licenses/>.
#

import os
import unittest
from unittest.mock import Mock

from ..policyrep.util import compile_policy

from setools.checker import PolicyChecker
from setools.exception import InvalidCheckerConfig, InvalidCheckerModule, InvalidCheckOption, \
    InvalidCheckValue


class PolicyCheckerTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.p = compile_policy("tests/checker/checker.conf")

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls.p.path)

    def test_config_empty(self):
        """Test empty config file"""
        with self.assertRaises(InvalidCheckerConfig):
            PolicyChecker(self.p, "/dev/null")

    def test_config_check_missing_type(self):
        """Test check missing check type"""
        with self.assertRaises(InvalidCheckerModule):
            PolicyChecker(self.p, "tests/checker/checker-missingtype.ini")

    def test_config_check_invalid_type(self):
        """Test check invalid check type"""
        with self.assertRaises(InvalidCheckerModule):
            PolicyChecker(self.p, "tests/checker/checker-invalidtype.ini")

    def test_config_check_invalid_option(self):
        """Test check invalid check option"""
        with self.assertRaises(InvalidCheckOption):
            PolicyChecker(self.p, "tests/checker/checker-invalidoption.ini")

    def test_config_check_invalid_value(self):
        """Test check invalid check type"""
        with self.assertRaises(InvalidCheckValue):
            PolicyChecker(self.p, "tests/checker/checker-invalidvalue.ini")

    def test_run_pass(self):
        """Test run with passing config."""
        with open(os.devnull, "w") as fd:
            checker = PolicyChecker(self.p, "tests/checker/checker-valid.ini")

            # create additional disabled mock test
            newcheck = Mock()
            newcheck.checkname = "disabled"
            newcheck.disable = True
            newcheck.validate_config.return_value = None
            newcheck.run.return_value = []
            checker.checks.append(newcheck)

            self.assertEqual(4, len(checker.checks))
            result = checker.run(output=fd)
            self.assertEqual(0, result)
            newcheck.run.assert_not_called()

    def test_run_fail(self):
        """Test run with failing config."""
        with open(os.devnull, "w") as fd:
            checker = PolicyChecker(self.p, "tests/checker/checker-valid.ini")

            # create additional failing mock test
            newcheck = Mock()
            newcheck.checkname = "failing test"
            newcheck.disable = False
            newcheck.validate_config.return_value = None
            newcheck.run.return_value = list(range(13))
            checker.checks.append(newcheck)

            self.assertEqual(4, len(checker.checks))

            result = checker.run(output=fd)
            newcheck.run.assert_called()
            self.assertEqual(13, result)



# Copyright 2020, Microsoft Corporation
#
# This file is part of SETools.
#
# SETools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1 of
# the License, or (at your option) any later version.
#
# SETools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with SETools.  If not, see
# <http://www.gnu.org/licenses/>.
#

import os
import unittest

from ..policyrep.util import compile_policy

from setools.checker.roexec import ReadOnlyExecutables
from setools.exception import InvalidCheckOption, InvalidCheckValue


class ReadOnlyExecutablesTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.p = compile_policy("tests/checker/roexec.conf")

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls.p.path)

    def test_invalid_option(self):
        """Test invalid option"""
        with self.assertRaises(InvalidCheckOption):
            config = {"INVALID": "option"}
            check = ReadOnlyExecutables(self.p, "test_invalid_option", config)

    def test_all_exec(self):
        """Test all executables are returned for no-option test.."""
        config = {}
        check = ReadOnlyExecutables(self.p, "test_all_exec", config)
        result = check._collect_executables()

        # becasue of unconfined, nonexec is executable
        expected = set(("roexec", "execfile1", "execfile2", "nonexec", "exempt_file"))
        self.assertSetEqual(expected, set(result.keys()))

    def test_exempt_exec_domain(self):
        """Test for exempting an exec domain."""
        config = {"exempt_exec_domain": "unconfined"}
        check = ReadOnlyExecutables(self.p, "test_exempt_exec_domain", config)
        result = check._collect_executables()

        expected = set(("execfile1", "execfile2", "roexec"))
        self.assertSetEqual(expected, set(result.keys()))

    def test_exempt_file(self):
        """Test for exempting a file."""
        config = {"exempt_file": "exempt_file"}
        check = ReadOnlyExecutables(self.p, "test_exempt_file", config)
        result = check._collect_executables()

        expected = set(("roexec", "execfile1", "execfile2", "nonexec"))
        self.assertSetEqual(expected, set(result.keys()))

    def test_exempt_file_attr(self):
        """Test for exempting a file by attribute."""
        config = {"exempt_file": "exempt_files_attr"}
        check = ReadOnlyExecutables(self.p, "test_exempt_file_attr", config)
        result = check._collect_executables()

        expected = set(("roexec", "nonexec", "exempt_file"))
        self.assertSetEqual(expected, set(result.keys()))

    def test_fail(self):
        """Test for failing."""
        with open("/dev/null", "w") as fd:
            config = {"exempt_exec_domain": "unconfined",
                      "exempt_write_domain": "unconfined"}
            check = ReadOnlyExecutables(self.p, "test_fail", config)
            check.output = fd
            result = check.run()

            expected = [self.p.lookup_type("execfile1"),
                        self.p.lookup_type("execfile2")]
            self.assertListEqual(expected, result)

    def test_pass(self):
        """Test for passing."""
        with open("/dev/null", "w") as fd:
            config = {"exempt_exec_domain": "unconfined",
                      "exempt_write_domain": "domain1  domain2  unconfined"}
            check = ReadOnlyExecutables(self.p, "test_pass", config)
            check.output = fd
            result = check.run()

            self.assertFalse(result)

    def test_pass2(self):
        """Test for passing with alternate exemptions."""
        with open("/dev/null", "w") as fd:
            config = {"exempt_exec_domain": "unconfined",
                      "exempt_file": "execfile2",
                      "exempt_write_domain": "domain1  unconfined"}
            check = ReadOnlyExecutables(self.p, "test_pass2", config)
            check.output = fd
            result = check.run()

            self.assertFalse(result)



# Copyright 2020, Microsoft Corporation
# Copyright 2020, Chris PeBenito <pebenito@ieee.org>
#
# This file is part of SETools.
#
# SETools is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 2.1 of
# the License, or (at your option) any later version.
#
# SETools is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with SETools.  If not, see
# <http://www.gnu.org/licenses/>.
#

import os
import unittest

from ..policyrep.util import compile_policy

from setools import RBACRuletype
from setools.checker.assertrbac import AssertRBAC
from setools.exception import InvalidCheckValue, InvalidCheckOption


class AssertRBACTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.p = compile_policy("tests/checker/assertrbac.conf")

    @classmethod
    def tearDownClass(cls):
        os.unlink(cls.p.path)

    def test_unconfigured(self):
        """Test unconfigured."""
        with self.assertRaises(InvalidCheckValue):
            config = {}
            check = AssertRBAC(self.p, "test_unconfigured", config)

    def test_invalid_option(self):
        """Test invalid option"""
        with self.assertRaises(InvalidCheckOption):
            config = {"INVALID": "option"}
            check = AssertRBAC(self.p, "test_invalid_option", config)

    def test_source(self):
        """Test source setting."""

        with self.subTest("Success"):
            config = {"source": "src"}
            check = AssertRBAC(self.p, "test_source", config)

            expected = self.p.lookup_role("src")
            self.assertEqual(expected, check.source)

        with self.subTest("Failure"):
            with self.assertRaises(InvalidCheckValue):
                config = {"source": "FAIL"}
                check = AssertRBAC(self.p, "test_source_fail", config)

    def test_target(self):
        """Test target setting."""

        with self.subTest("Success"):
            config = {"target": "tgt"}
            check = AssertRBAC(self.p, "test_target", config)

            expected = self.p.lookup_role("tgt")
            self.assertEqual(expected, check.target)

        with self.subTest("Failure"):
            with self.assertRaises(InvalidCheckValue):
                config = {"target": "FAIL2"}
                check = AssertRBAC(self.p, "test_target_fail", config)

    def test_exempt_source(self):
        """Test exempt_source setting."""
        with self.subTest("Success"):
            config = {"source": "system",
                      "exempt_source": " exempt_src1   exempt_src2 "}
            check = AssertRBAC(self.p, "test_exempt_source", config)

            # exempt_src2 is an attr
            expected = set((self.p.lookup_role("exempt_src1"),
                            self.p.lookup_role("exempt_src2")))
            self.assertIsInstance(check.exempt_source, frozenset)
            self.assertSetEqual(expected, check.exempt_source)

        with self.subTest("Success, missing role ignored"):
            """Test exempt_source missing role is ignroed."""
            config = {"source": "system",
                      "exempt_source": "FAIL  exempt_src2"}
            check = AssertRBAC(self.p, "test_source_missing_ignored", config)

            # exempt_src2 is an attr
            expected = set((self.p.lookup_role("exempt_src2"),))
            self.assertIsInstance(check.exempt_source, frozenset)
            self.assertSetEqual(expected, check.exempt_source)

    def test_exempt_target(self):
        """Test exempt_target setting."""
        with self.subTest("Success"):
            config = {"target": "system",
                      "exempt_target": " exempt_tgt1   exempt_tgt2 "}
            check = AssertRBAC(self.p, "test_exempt_target", config)

            # exempt_tgt2 is an attr
            expected = set((self.p.lookup_role("exempt_tgt1"),
                            self.p.lookup_role("exempt_tgt2")))
            self.assertIsInstance(check.exempt_target, frozenset)
            self.assertSetEqual(expected, check.exempt_target)

        with self.subTest("Success, missing role ignored"):
            config = {"target": "system",
                      "exempt_target": "FAIL  exempt_tgt2"}
            check = AssertRBAC(self.p, "test_target_missing_ignored", config)

            # exempt_tgt2 is an attr
            expected = set((self.p.lookup_role("exempt_tgt2"),))
            self.assertIsInstance(check.exempt_target, frozenset)
            self.assertSetEqual(expected, check.exempt_target)

    def test_expect_source(self):
        """Test expect_source setting."""
        with self.subTest("Success"):
            config = {"target": "tgt",
                      "expect_source": " exempt_src1   exempt_src2 "}
            check = AssertRBAC(self.p, "test_expect_source", config)

            # exempt_src2 is an attr
            expected = set((self.p.lookup_role("exempt_src1"),
                            self.p.lookup_role("exempt_src2")))
            self.assertIsInstance(check.expect_source, frozenset)
            self.assertSetEqual(expected, check.expect_source)

        with self.subTest("Failure"):
            with self.assertRaises(InvalidCheckValue):
                config = {"target": "tgt",
                          "expect_source": " source1   INVALID "}
                check = AssertRBAC(self.p, "test_expect_source_fail", config)

    def test_expect_target(self):
        """Test expect_target setting."""
        with self.subTest("Success"):
            config = {"source": "src",
                      "expect_target": " exempt_tgt1   exempt_tgt2 "}
            check = AssertRBAC(self.p, "test_expect_target", config)

            # exempt_tgt2 is an attr
            expected = set((self.p.lookup_role("exempt_tgt1"),
                            self.p.lookup_role("exempt_tgt2")))
            self.assertIsInstance(check.expect_target, frozenset)
            self.assertSetEqual(expected, check.expect_target)

        with self.subTest("Failure"):
            with self.assertRaises(InvalidCheckValue):
                config = {"source": "src",
                          "expect_target": " target1   INVALID "}
                check = AssertRBAC(self.p, "test_expect_target_fail", config)

    def test_check_passes(self):
        """Test the check passes, no matches"""
        config = {"source": "src",
                  "target": "tgt"}
        check = AssertRBAC(self.p, "test_check_passes", config)
        self.assertFalse(check.run())

    def test_check_passes_exempt_source_role(self):
        """Test the check passes, exempt_source_role"""
        config = {"target": "target1",
                  "exempt_source": "source1"}
        check = AssertRBAC(self.p, "test_check_passes_exempt_source_role", config)
        self.assertFalse(check.run())

    def test_check_passes_exempt_target_role(self):
        """Test the check passes, exempt_target_role"""
        config = {"target": "target2",
                  "exempt_source": "source2"}
        check = AssertRBAC(self.p, "test_check_passes_exempt_target_role", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_source(self):
        """Test the check passes, expect_source"""
        config = {"target": "target3",
                  "expect_source": "source3a source3b"}
        check = AssertRBAC(self.p, "test_check_passes_expect_source", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_target(self):
        """Test the check passes, expect_target"""
        config = {"source": "source4",
                  "expect_target": "target4a target4b"}
        check = AssertRBAC(self.p, "test_check_passes_expect_target", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_exempt_source(self):
        """"Test the check passes with both expected and exempted sources."""
        config = {"target": "target5",
                  "expect_source": "source5a",
                  "exempt_source": "source5b"}
        check = AssertRBAC(self.p, "test_check_passes_expect_exempt_source", config)
        self.assertFalse(check.run())

    def test_check_passes_expect_exempt_target(self):
        """"Test the check passes with both expected and exempted targets."""
        config = {"source": "source6",
                  "expect_target": "target6a",
                  "exempt_target": "target6b"}
        check = AssertRBAC(self.p, "test_check_passes_expect_exempt_target", config)
        self.assertFalse(check.run())

    def test_check_fails(self):
        """Test the check fails"""
        with open("/dev/null", "w") as fd:
            config = {"source": "source7",
                      "expect_target": "target7a",
                      "exempt_target": "target7b"}
            check = AssertRBAC(self.p, "test_check_passes_exempt_target_attr", config)
            check.output = fd
            result = check.run()
            self.assertEqual(1, len(result), msg=result)
            rule = result[0]
            self.assertEqual(RBACRuletype.allow, rule.ruletype)
            self.assertEqual("source7", rule.source)
            self.assertEqual("target7c", rule.target)

    def test_check_fails_expect_source(self):
        """Test the check fails, expect_source"""
        config = {"target": "target8",
                  "expect_source": "source8"}
        check = AssertRBAC(self.p, "test_check_fails_expect_source", config)
        result = check.run()
        self.assertEqual(1, len(result), msg=result)
        self.assertIn("source8", result[0])

    def test_check_fails_expect_target(self):
        """Test the check fails, expect_target"""
        config = {"source": "source9",
                  "expect_target": "target9"}
        check = AssertRBAC(self.p, "test_check_fails_expect_target", config)
        result = check.run()
        self.assertEqual(1, len(result), msg=result)
        self.assertIn("target9", result[0])
